import time, os, json, ctypes
from PyQt5 import QtCore
import myutil

class CDWriterS38Robot(QtCore.QObject):
	signalUI = QtCore.pyqtSignal(int, str)
	DEV_PORT = 6000
	if os.sep == '/': # linux
		libPath = '/path/libYourCpp.so'
	else:
		libPath = os.path.dirname(__file__) + '/yourCpp.dll'
	initState = False # 是否已初始化

	def __init__(self, devName, mainConfig):
		'''Class object init
		:param devName: 设备名PrinterName returned by enumDevice
		:param mainConfig: dict 刻录相关的设置 secu-dk.json
		'''
		super(QtCore.QObject, self).__init__()
		self.__configJson = mainConfig
		self.__devAddress = None
		self.totalCopy = 0
		self.indexCopy = 0
		self.taskState = 0
		self.lib = None
		self.clientHandle = None
		self.taskHandle = None
		self.taskId = None

	@staticmethod
	def enumDevice(devInfo):
		''' 枚举所有已连接的设备。该函数在：管理员注册刻录机、用户进入刻录任务窗口时调用一次。
		:param devInfo: 注册刻录机时用户输入的信息，如IP地址等
		:return: 错误信息如 无法连接USB设备 等，成功则为数组 [{'PrinterName':主机名(设备名), 'pbyUniqueID':devUUID}]
		设备名为设备的名称型号，可以重复；pbyUniqueID为该设备的唯一ID, 在initDevice会传入，与其他同型号设备不能一样。
		'''
		print('enumDevice', devInfo)
		time.sleep(2)
		try:
			if False:
				return '未连接刻录设备'
			devName = '%s(%s)' % (myutil.host['Name'], 'SBP-38-Robot')
			devInfo = 'abcdef-098765-4321'
		except Exception as e:
			return 'enumDevice exp:'+repr(e)

		return [{'PrinterName':devName, 'pbyUniqueID':devInfo}]

	def initDevice(self, devAddress, totalCopy):
		'''设备初始化，如机械复位、清理废盘等。所有刻录任务开始前调用一次。
		:param devAddress: 设备唯一ID(pbyUniqueID returned by enumDevice)
		:param totalCopy: 本次所有任务的总份数
		:reutrn: None, or error message
		'''
		print('initDevice', devAddress, totalCopy)
		self.__devAddress = devAddress
		self.totalCopy = totalCopy
		if CDWriterS38Robot.initState: # 已经初始化
			return None

		errMsg = None
		if False:
			errMsg = '无法连接设备、空盘数量不够……'
		time.sleep(2)
		CDWriterS38Robot.initState = True
		return errMsg

	def beginTask(self, dataFile, cdType, nCopy, listDiscImage):
		'''设备初始化，如机械复位、清理废盘等。每个刻录任务开始前调用一次。
		:param dataFile: 待刻录文件夹或 iso 路径
		:param cdType: 光盘类型 CD=1,DCD=2,BlueRayDVD=3
		:param nCopy: 当前任务的份数
		:param listDiscImage: 每个盘面的图片，如['/path/a1.png','/path/a2.png']
		:reutrn: None, or error message
		'''
		print('beginTask', dataFile, cdType, nCopy, listDiscImage)
		time.sleep(2)
		return None

	def prepareMedia(self, sizeM, cdType):
		'''准备光盘。每份光盘调用一次。
		:param sizeM: 刻录内容大小(M), 如 1000
		:param cdTypeNum: 光盘类型 CD=1,DCD=2,BlueRayDVD=3
		:reutrn: None, or error message
		'''
		print('prepareMedia', sizeM, cdType)
		if False:
			return '没有光盘、容量不足'
		time.sleep(2)
		return None

	def printDisk(self, discImg):
		'''打印盘面，每份光盘调用一次。
		:param discImg: 图片文件路径如 /var/lib/1234.png
		:reutrn: None, or error message
		'''
		print('printDisk', discImg)
		time.sleep(2)
		return None

	def writeData(self, dataFile):
		'''写入光盘。刻录任务开始前调用一次。
		:param dataFile: 待刻录文件夹或 iso 路径
		:reutrn: None, or error message
		'''
		print('writeData', dataFile)
		time.sleep(2)
		self.signalUI.emit(1, '正在刻录……25%')
		return None

	def completeMedia(self):
		'''从光驱或打印架上取走光盘。每份刻录完成，或光盘不符合要求时调用一次。
		'''
		print('completeMedia')
		time.sleep(2)
		return None

	def endTask(self):
		'''结束任务时调用一次
		'''
		print('endTask')
		time.sleep(2)
		return None

	def freeDevice(self, succ):
		'''释放设备，如清理废盘等。任务结束时调用一次。
		:param succ: 1=成功 0=失败 两种情况的清理操作可能不同
		'''
		print('freeDevice', succ)
		time.sleep(2)
		return None

if __name__ == '__main__':
	myutil.getHostIpMac()
	myutil.appPath = os.path.dirname(__file__)
	#myutil.initLogger('C:/Windows/Temp', 'test_s38', 4)

	listDev = CDWriterS38Robot.enumDevice('')
	devName = listDev[0]['PrinterName']
	devUUID = listDev[0]['pbyUniqueID']
	dictConfig = {'CloseMedia':1, 'CDWriteDummy':0}
	print(devName, devUUID, '\n')
	cdw = CDWriterS38Robot(devName, dictConfig)

	errMsg = cdw.initDevice(devUUID, 1)
	listTask = [{'name':'task-a','data':'/tmp/a.iso','copy':1, 'img':['/tmp/a1.png']}, 
		{'name':'task-b','data':'/tmp/b.iso','copy':2, 'img':['/tmp/b1.png','/tmp/b2.png']}]
	for task in listTask:
		print('begin', task['name'])
		cdw.beginTask(task['data'], 1, 1, task['img'])
		for i in range(task['copy']):
			cdw.prepareMedia(500, 1)
			cdw.printDisk(task['img'][i])
			cdw.writeData(task['data'])
			cdw.completeMedia()
		cdw.endTask()
		print(task['name'], 'end\n')
	cdw.freeDevice(0)
